/*  gtk-imonc - imond client for fli4l
 *  Copyright (C) 2001 Stefan Strigler <zeank@x-berg.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "imonc.h"
#include "callbacks.h"
#include "timer.h"

void show_usage(char *argv[])
{
	fprintf (stderr, "usage: %s [-n] [-p pass] hostname [port]\n", argv[0]);
	fprintf (stderr, "  whereas:\n");
	fprintf (stderr, "  -n\t\tdon't display news panel\n");
	fprintf (stderr, "  -p pass\tuse pass as password\n");
	exit(1);
}

int
main (int argc, char *argv[])
{
	GtkWidget *auth_dialog;
	char *answer;
	gchar buf[64];
	gchar *pass = NULL;
	struct sigaction act;
	guint arg = 1;

	port = IMOND_PORT;

	siginterrupt (SIGALRM, 1);

	/* Assign sig_pipe as SIGPIPE handler */
	act.sa_handler = fli4l_reconnect;

	/* We don't want to block any other signals */
	sigemptyset(&act.sa_mask);

	if (sigaction(SIGPIPE, &act, NULL) < 0) {
		fprintf(stderr, "sigaction failed\n");
		return 1;
	}

	signal (SIGALRM, fli4l_reconnect);

#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
	add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

	/* parse command line arguments
	 * proposed format:
	 * $ gtk-imonc [-n] [-p pass] hostname [port]
	 */

	if (argc < 2) 
		show_usage(argv);

	use_news_tab = TRUE;
	if (strcmp (argv[arg], "-n") == 0)
		{
			use_news_tab = FALSE;
			arg++;
		}

	if (argc<=arg)
		show_usage(argv);

	if (strcmp (argv[arg], "-p") == 0)
		{
			arg++;
			if (argc <= arg)
				show_usage(argv);
			pass = argv[arg];
			arg++;
		}

	if (argc <= arg)
		show_usage(argv);

	host_name = argv[arg++];

	if (arg < argc)
		port = atoi (argv[arg]);

	if ((fd = service_connect (host_name, port)) < 0) {
		exit (1);
	}


/* 	halt_warning = create_halt_warning (); */
/* 	reboot_warning = create_reboot_warning (); */

	gtk_imonc = create_gtk_imonc ();

	/* check password */
	if (pass != NULL)
		{
			sprintf (buf, "pass %s", pass);
			send_command (fd, buf);
		}
	else
		{
			send_command (fd, "pass");
		}
	answer = get_answer (fd);
			
	/* 	From Nico Wallmeier:
	 *		1: Userpasswort ntig
	 * 		2: Adminpasswort ntig
	 * 		3: User+ Adminpasswort ntig
	 * 		+4 im Adminmodus 
	 */

	if (answer && ((strcmp (answer, "1")==0) || (strcmp (answer, "3")==0))) { /* need to check password */
		auth_dialog = create_auth_dialog ();
		gtk_object_set_data (GTK_OBJECT (auth_dialog), "gtk_imonc", gtk_imonc);
		gtk_widget_show (auth_dialog);
	} else {
		
		imonc_init (gtk_imonc); /* initialize imonc functionality */
		imonc_update (gtk_imonc); /* init with current values */
		
		timeout_tag = gtk_timeout_add (TIMEOUT_MSEC, imonc_update, gtk_imonc);
		gtk_widget_show (gtk_imonc);
		
	}
	
	gtk_main ();
	
	service_disconnect (fd);
	return 0;
}
