/*  gtk-imonc - imond client for fli4l
 *  Copyright (C) 2001 Stefan Strigler <zeank@x-berg.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "imonc.h"
#include "timer.h"

#include <time.h>

/* set titles of calls tab */
void init_calls (GtkCList *clist)
{
	gtk_clist_set_column_title (clist, 0, "Date");	
	gtk_clist_set_column_title (clist, 1, "Time");
	gtk_clist_set_column_title (clist, 2, "Calling Number");
	gtk_clist_set_column_title (clist, 3, "Called Number");
}

/* set titles for connections tab */
void init_imondlog (GtkCList *clist)
{
	gtk_clist_set_column_title (clist, 0, "Circuit");
	gtk_clist_set_column_title (clist, 1, "StartDate");
	gtk_clist_set_column_title (clist, 2, "StartTime");
	gtk_clist_set_column_title (clist, 3, "EndDate");
	gtk_clist_set_column_title (clist, 4, "EndTime");
	gtk_clist_set_column_title (clist, 5, "OnlTime");
	gtk_clist_set_column_title (clist, 6, "ChTime");
	gtk_clist_set_column_title (clist, 7, "Charge");
	gtk_clist_set_column_title (clist, 8, "IBytes");
	gtk_clist_set_column_title (clist, 9, "OBytes");
	gtk_clist_set_column_title (clist, 10, "Device");
	gtk_clist_set_column_title (clist, 11, "ChInt");
	gtk_clist_set_column_title (clist, 12, "Ch/Min");
}

/*----------------------------------------------------------------------------
 * get timetable and fill tab
 *----------------------------------------------------------------------------
 */
void
get_timetable														(GtkCList *timetable_clist, 
																				 GtkCList *circuits_clist)
{
	GSList *list;
	gpointer data;
	gchar *timetable_row[26];
	gchar *circuits_row[7];
	guint i,j;

	list = get_list (fd, "timetable");

	if (list == NULL)
		return;

	for (i=2;i<9; i++) { 
		data = g_slist_nth_data (list, i);

		timetable_row[0] = strtok (data, " ");
		for (j=1; j<25; j++)
			timetable_row[j] = strtok (NULL, " ");
		/* last col is empty */
		timetable_row[25] = "";
		gtk_clist_append (timetable_clist, timetable_row);
		g_free(data);
	}


	list = g_slist_nth (list, 10);

	while ((list = g_slist_next (list)) != NULL) {
		circuits_row[0] = strtok (list->data, " ");
		for (j=1; j<7; j++)
			circuits_row[j] = strtok (NULL, " ");

		gtk_clist_append (circuits_clist, circuits_row);
		g_free(list->data);
	}
	g_slist_free(list);
}

/*----------------------------------------------------------------------------
 * get telmondlog file from imond and fill clist
 *----------------------------------------------------------------------------
 */
void 
get_calls																(GtkCList *calls_clist)
{
	GSList *list;
	gchar *row[4];

	list = get_list (fd, "telmond-log-file");

	if (list == NULL)
		return;

	do {
		row[0] = strtok (list->data, " ");
		row[1] = strtok (NULL, " ");
		row[2] = strtok (NULL, " ");
		row[3] = strtok (NULL, " ");
		gtk_clist_prepend (calls_clist, row);
		g_free (list->data);
	} while ((list = g_slist_next (list)) != NULL);

	g_slist_free (list); /* free up memory */
}

/*----------------------------------------------------------------------------
 * get imondlog from imond and fill clist
 *----------------------------------------------------------------------------
 */
void
get_connections													(GtkCList *imondlog_clist)
{
	GSList *list;
	gchar *row[IMONDLOG_CLIST_COLS];
	guint i, j , tmpbytes;
	gchar ibytebuf[64], obytebuf[64], \
		chargebuf[64], onl_time_buf[64], ch_time_buf[64];

	GtkWidget *imondlog_onltime_label, *imondlog_ibytes_label, \
		*imondlog_obytes_label, *imondlog_charge_label;

	imondlog_onltime_label = lookup_widget (GTK_WIDGET (imondlog_clist), "imondlog_onltime_label");
	imondlog_ibytes_label = lookup_widget (GTK_WIDGET (imondlog_clist), "imondlog_ibytes_label");
	imondlog_obytes_label = lookup_widget (GTK_WIDGET (imondlog_clist), "imondlog_obytes_label");
	imondlog_charge_label = lookup_widget (GTK_WIDGET (imondlog_clist), "imondlog_charge_label");

	list = get_list (fd, "imond-log-file");

	if (list == NULL)
		return;

	j=0; /* count lines */
	do {
		if (j++ < con_rows) { /* only prepend new lines */
			g_free(list->data); /* free up used string elem */
			continue;
		} else {
			con_rows++;
		}
		row[0] = strtok (list->data, " ");
		for (i=1; i<IMONDLOG_CLIST_COLS; i++) {
			switch (i) { /* split it up here for special treatment */
			case 5: /* OnlTime */
				strcpy (onl_time_buf, strtok (NULL, " "));
				onltime += atoi (onl_time_buf);
				row[i] = get_timestr (atoi (onl_time_buf), onl_time_buf);
				break;
			case 6: /* ChTime */
				row[i] = get_timestr (atoi (strtok (NULL, " ")), ch_time_buf);
				break;
			case 7: /* Charge */
				strcpy (chargebuf, strtok (NULL, " "));
				row[i] = chargebuf;
				break;
			case 8: /* IBytes */
				tmpbytes = atoi (strtok (NULL, " "));
				ibytes += tmpbytes;
				row[i] = get_bytes (tmpbytes, ibytebuf);
				break;
			case 9: /* OBytes */
				tmpbytes = atoi (strtok (NULL, " "));
				obytes += tmpbytes;
				row[i] = get_bytes (tmpbytes, obytebuf);
				break;
			default:
				row[i] = strtok (NULL, " ");
			}
		}
		/* add the row */
		gtk_clist_prepend (imondlog_clist, row);

		/* split up charge string 
		 * we must do this here cause strtok uses a static buffer
		 */
		charge += ((atoi (strtok (chargebuf, "."))) * 100) + (atoi (strtok (NULL, ".")));
		g_free(list->data); /* free up used string elem */

	} while ((list = g_slist_next (list)) != NULL);

	g_slist_free(list); /* free up useless list */

	/* autosize the whole list */
	gtk_clist_columns_autosize (imondlog_clist);

	/* set labels of summaries */
	gtk_label_set_text (GTK_LABEL (imondlog_onltime_label), get_timestr (onltime, onl_time_buf));

	gtk_label_set_text (GTK_LABEL (imondlog_ibytes_label), get_bytes (ibytes, ibytebuf));

	gtk_label_set_text (GTK_LABEL (imondlog_obytes_label), get_bytes (obytes, obytebuf));

	sprintf (chargebuf, "%d,%d", (charge/100), (charge%100));
	gtk_label_set_text (GTK_LABEL (imondlog_charge_label), chargebuf);

} /* end get_connections */

/*----------------------------------------------------------------------------
 * gets news from fli4l.de
 *----------------------------------------------------------------------------
 */
void 
get_news																(GtkText *news_text)
{
	static gchar *buf;
	static gint  buffer_size = 0;
	gchar        ch[1];
	gint         web_fd;
	gint         n = 0;
	gchar        **strarr;

	if ((web_fd = service_connect ("fli4l.de", 80)) < 0)
		return;

	send_command (web_fd, "GET http://fli4l.de/german/news.txt / HTTP/1.1\n\n");

	if (buffer_size == 0)
		{
			buffer_size = GRANULARITY;
			buf = g_malloc (buffer_size);
		}

	while (read (web_fd, ch, 1) > 0) {
		if (n + 1 == buffer_size)
			{
				buffer_size += GRANULARITY;
				buf = realloc (buf, buffer_size);
			}
		buf[n] = ch[0];
		n++;
	}

	/* init text box */
	/* avoid flickering */
	gtk_text_freeze (GTK_TEXT (news_text));

	/* clear box */
	gtk_text_set_point (GTK_TEXT (news_text), 0);
	gtk_text_forward_delete (GTK_TEXT (news_text), gtk_text_get_length (GTK_TEXT (news_text)));

	/* buffer ready - now parse it */
	strarr = g_strsplit (buf, "\n", 0);

	/* skip http header */
	while (*strarr != NULL) {
		if (*strarr[0] == '\r')
			break;
		strarr++;
	}

	strarr ++;
	while (*strarr != NULL) {
		sprintf (*strarr, "%s\n", *strarr);
		gtk_text_insert (news_text, NULL, NULL, NULL, *strarr, -1);
		strarr++;
	}

	/* release box */
	gtk_text_thaw (GTK_TEXT (news_text));

	service_disconnect (web_fd);
} /* end get_news */


/*----------------------------------------------------------------------------
 * initializes some settings of the interface 
 *----------------------------------------------------------------------------
 */
void imonc_init (gpointer window)
{
	gint i;
	char buf[64];
	gchar *clist_row[ROW_LENGTH];
	gchar about[128];
	GtkWidget *dial_button, *hangup_button, *reboot_button, \
		*halt_button, *remlink_button, \
		*default_route_menu, *default_route_menu_menu, \
		*dialmode, *new_menu_item, *clist, \
		*timetable_clist, *circuits_clist, *calls_clist, *imondlog_clist, \
		*notebook, *about_label, *version_error;
	gint imondlog_tab_nr;
	
	/* get button */
	dial_button = lookup_widget (window, "dial_button");
	hangup_button = lookup_widget (window, "hangup_button");
	remlink_button = lookup_widget (window, "remlink_button");
	reboot_button = lookup_widget (window, "reboot_button");
	halt_button = lookup_widget (window, "halt_button");
	default_route_menu = lookup_widget (window, "default_route_menu");
	default_route_menu_menu = GTK_OPTION_MENU (default_route_menu)->menu;
	dialmode = lookup_widget (window, "dialmode");
	clist = lookup_widget (window, "channel_clist");
	timetable_clist = lookup_widget (window, "timetable_clist");
	circuits_clist = lookup_widget (window, "circuits_clist");
	calls_clist = lookup_widget (window, "calls_clist");
	imondlog_clist = lookup_widget (window, "imondlog_clist");
	notebook = lookup_widget (window, "notebook");
	about_label = lookup_widget (window, "about_label");

	/* init icons */
	online_icon = create_gdk_pixmap (window, "online.xpm", &online_icon_mask);
	offline_icon = create_gdk_pixmap (window, "offline.xpm", &offline_icon_mask);

	/* get imond version */
	send_command (fd, "version");
	imond_versionnr = atoi (strtok (get_answer (fd), " "));
	strcpy (imond_version, strtok (NULL, " "));

	/* check version and abort if imond to old */
	if (imond_versionnr < 10) {
		g_print ("imond to old\n");
		version_error = create_version_error ();
		gtk_widget_show (version_error);
		return;
	}

	/* get imond hostname - this might be different from connected host
		 due to used tunnels or so */
	send_command (fd, "hostname");
	sprintf (imond_hostname, "%s", get_answer (fd));

	/* check which functions are available */
	send_command (fd, "is-allowed dial");
	if(! (atoi (get_answer (fd)))) {
		gtk_widget_set_sensitive (dial_button, FALSE);
		gtk_widget_set_sensitive (hangup_button, FALSE);
	}
	
	send_command (fd, "is-allowed reboot");
	if(! (atoi (get_answer (fd)))) {
		gtk_widget_set_sensitive (reboot_button, FALSE);
		gtk_widget_set_sensitive (halt_button, FALSE);
	}
	
	send_command (fd, "is-allowed route");
	if(! (atoi (get_answer (fd))))
		gtk_widget_set_sensitive (default_route_menu, FALSE);
	
	gtk_widget_hide (remlink_button);
	
	/* setting up menu for circuits (default route) */
	send_command (fd, "circuits");
	circuits = atoi (get_answer (fd));
	
	for (i=1; i<=circuits; i++) {
		sprintf (buf, "circuit %i", i);
		send_command (fd, buf);
		
		new_menu_item = gtk_menu_item_new_with_label (get_answer (fd));
		gtk_widget_show (new_menu_item);
		gtk_menu_append (GTK_MENU (default_route_menu_menu), new_menu_item);
	}

	/* clear list to be sure */
	gtk_clist_clear (GTK_CLIST (clist));
	
	/* make a clean row for later usage */
	for (i=0; i<ROW_LENGTH; i++) {
		clist_row[i] = "";
	}

	send_command (fd, "pppoe");
	pppoe = atoi (get_answer (fd));

	if (pppoe) {
		/* add a row for pppoe */
		gtk_clist_append (GTK_CLIST (clist), clist_row);
	}

	send_command (fd, "channels");
	channels = atoi (get_answer (fd));
	for (i=1; i<=channels; i++) {
	  /* foresee a clist_row for every circuit */
	  gtk_clist_append (GTK_CLIST (clist), clist_row);
	}
	
	gtk_signal_connect (GTK_OBJECT (default_route_menu_menu), "deactivate", GTK_SIGNAL_FUNC (on_default_route_selected), NULL); 
	
	/* setting up menu for dialmode */
	gtk_signal_connect (GTK_OBJECT (GTK_OPTION_MENU (dialmode)->menu), "deactivate", GTK_SIGNAL_FUNC (on_dialmode_selected), NULL);
	
	/* init timetable */
	get_timetable (GTK_CLIST (timetable_clist), GTK_CLIST (circuits_clist));

	/* check if we should display the news tab */
	if (!use_news_tab)
		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), 4);

	/* init calls */
	imondlog_tab_nr = 2; 
	send_command (fd, "is-allowed telmond-log");
	if (atoi (get_answer (fd))) {
		init_calls(GTK_CLIST (calls_clist));
		get_calls (GTK_CLIST (calls_clist));
	} else {
		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), 1);
		imondlog_tab_nr = 1;
	}

	/* init connections overview */
	send_command (fd, "is-allowed imond-log");
	if (atoi (get_answer (fd))) {
		init_imondlog(GTK_CLIST (imondlog_clist));
		con_rows = onltime = obytes = ibytes = charge = 0; /* init global values */
		gtk_clist_column_titles_passive (GTK_CLIST (imondlog_clist));
		get_connections (GTK_CLIST (imondlog_clist));
		imond_log = TRUE;
	} else {
		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), imondlog_tab_nr);
		imond_log = FALSE;
	}

	/* insert version nr. into about label */
	sprintf (about, _("\nGTK-Imonc\n\nVersion %s\n\nCopyright (c) 2001 by Stefan Strigler <zeank@x-berg.de>\n\nhttp://userpage.fu-berlin.de/~zeank/gtk-imonc/"), VERSION);
	gtk_label_set_text (GTK_LABEL (about_label), about);

	/* make sure status is shown at startup */
	show_status = TRUE;
	init_show_status = TRUE;

	/* set news init */
	news_init = 1;

} /* imonc_init (gpointer window) */

/*---------------------------------------------------------------------------- 
 * this is THE timer function called every TIMEOUT_MSEC msec
 * gets data from router and updates interface accordingly
 *----------------------------------------------------------------------------
 */
gint imonc_update (gpointer window)
{
	guint i;
	gchar buf[64];

	GtkWidget *dial_button, *hangup_button, \
		*addlink_button, *remlink_button, \
		*admin_mode_button, *calls_reset_button, *imondlog_reset_button, \
		*imondlog_clist, *config_circuits_button, *news_text;
	
	/* get ui elements */
	dial_button = lookup_widget (window, "dial_button");
	hangup_button = lookup_widget (window, "hangup_button");
	addlink_button = lookup_widget (window, "addlink_button");
	remlink_button = lookup_widget (window, "remlink_button");
	admin_mode_button = lookup_widget (window, "admin_mode_button");
	calls_reset_button = lookup_widget (window, "calls_reset_button");
	imondlog_reset_button = lookup_widget (window, "imondlog_reset_button");
	imondlog_clist = lookup_widget (window, "imondlog_clist");
	config_circuits_button = lookup_widget (window, "config_circuits_button");
	news_text = lookup_widget (window, "news_text");

	is_online = FALSE;

	/* get online-status */
	if (pppoe) { /* DSL */
		send_command (fd, "status pppoe");
		if (strcmp (get_answer (fd), "Online") == 0) {
			/* we are online */
			is_online = TRUE; /* remember online-status */
		}
	}
	
	for(i=1; i<=channels; i++) { /* ISDN */
		sprintf (buf, "status %i", i);
		send_command (fd, buf);
		if (strcmp (get_answer (fd), "Online") == 0) {
			is_online = TRUE; /* remember state */
		}
	}

	if (show_status) /* true if status page is visible */
		status_display_update (window);

	/* set dial|hangup button */
	if (is_online) {
		gtk_widget_hide (dial_button);
		gtk_widget_show (hangup_button);
		was_online = TRUE; /* remember again */
		if (news_init && use_news_tab) {
			get_news (GTK_TEXT (news_text));
			news_init = 0;
		}
	} else {
		gtk_widget_hide (hangup_button);
		gtk_widget_show (dial_button);

		if (was_online) {
			was_online = FALSE;
			/* update connections overview */
			if (imond_log)
				get_connections (GTK_CLIST (imondlog_clist));
		}
	}

	/* set admin-mode button */
	send_command (fd, "pass");
	if ((atoi (get_answer (fd))) >= 4 ) {
		gtk_widget_set_sensitive (admin_mode_button, FALSE);
		gtk_widget_set_sensitive (calls_reset_button, TRUE);
		gtk_widget_set_sensitive (imondlog_reset_button, TRUE);
		gtk_widget_set_sensitive (config_circuits_button, TRUE);

		/* set channel-bundling stuff */
		send_command (fd, "links 0");
		
		switch (atoi (get_answer (fd))) {
		case 1:
			gtk_widget_set_sensitive (addlink_button, TRUE);
			gtk_widget_hide (remlink_button);
			gtk_widget_show (addlink_button);
			break;
		case 2:
			gtk_widget_set_sensitive (remlink_button, TRUE);
			gtk_widget_hide (addlink_button);
			gtk_widget_show (remlink_button);
			break;
		default:
			gtk_widget_set_sensitive (addlink_button, FALSE);
			gtk_widget_set_sensitive (remlink_button, FALSE);
		}

	} else {
		gtk_widget_set_sensitive (addlink_button, FALSE);
		gtk_widget_set_sensitive (calls_reset_button, FALSE);
		gtk_widget_set_sensitive (imondlog_reset_button, FALSE);
		gtk_widget_set_sensitive (config_circuits_button, FALSE);
	}

 	return 1;
} /* imonc_update (gpointer window) */

/*----------------------------------------------------------------------------
 * makes update to values set by imonc_init
 * called when user switches to admin-mode
 *----------------------------------------------------------------------------
 */
void
imonc_auth_update (gpointer window)
{
	GtkWidget *dial_button, *hangup_button, \
		*reboot_button, *halt_button, \
		*default_route_menu, *support_button;

	dial_button = lookup_widget (window, "dial_button");
	hangup_button = lookup_widget (window, "hangup_button");
	reboot_button = lookup_widget (window, "reboot_button");
	halt_button = lookup_widget (window, "halt_button");
	default_route_menu = lookup_widget (window, "default_route_menu");
	support_button = lookup_widget (window, "support_button");

	/* check which functions are available */
	send_command (fd, "is-allowed dial");
	if(atoi (get_answer (fd))) {
		gtk_widget_set_sensitive (dial_button, TRUE);
		gtk_widget_set_sensitive (hangup_button, TRUE);
	}

	send_command (fd, "is-allowed reboot");
	if(atoi (get_answer (fd))) {
		gtk_widget_set_sensitive (reboot_button, TRUE);
		gtk_widget_set_sensitive (halt_button, TRUE);
	}

	send_command (fd, "is-allowed route");
	if(atoi (get_answer (fd)))
		gtk_widget_set_sensitive (default_route_menu, TRUE);

	gtk_widget_set_sensitive (support_button, TRUE);

} /* imonc_auth_update (gpointer window) */


/*----------------------------------------------------------------------------
 * updates items on "status" notebook page
 *----------------------------------------------------------------------------
 */
void
status_display_update (gpointer window) {
	guint i, j;
	gchar buf[64];
	gchar *answer;
	gchar answers[5][8192];

	GtkWidget *clist, *remote_date_label, \
		*default_route_menu, *dialmode, *lastcall_label;

	/* get ui elements */
	clist = lookup_widget (window, "channel_clist");
	remote_date_label = lookup_widget (window, "remote_date_label");
	default_route_menu = lookup_widget (window, "default_route_menu");
	dialmode = lookup_widget (window, "dialmode");
	lastcall_label = lookup_widget (window, "lastcall_label");
	
	i = j = 0;

	/* updating remote machine status */
	send_command (fd, "date"); /* get date */
	strcpy (answers[1], get_answer (fd));

	if (imond_versionnr >= 8) {
		/* only newer version support cpu and uptime status */
		send_command (fd, "uptime");
		strcpy (answers[3], get_answer (fd));
		send_command (fd, "cpu");
		strcpy (answers[4], get_answer (fd));
		sprintf (answers[0], _("[ %s - v%s ]\ndate: %s, uptime: %s, cpu: %s%%"), imond_hostname, imond_version, answers[1], get_datestr (atoi (answers[3]), buf), answers[4]);
	} else {
		/* older versions just display date */
		sprintf (answers[0], _("[ %s - v%s ]\ndate: %s"), imond_hostname, imond_version, answers[1]);
	}

	gtk_label_set_text (GTK_LABEL(remote_date_label), answers[0]);

	/* set dialmode */
	send_command (fd, "dialmode");
	strcpy (buf, get_answer (fd));
	if (strcmp (buf, "auto") == 0)
		i = 0;
	else if (strcmp (buf, "manual") == 0)
		i = 1;
	else if (strcmp (buf, "off") == 0)
		i = 2;
	gtk_option_menu_set_history (GTK_OPTION_MENU (dialmode), i);

	/* set default route menu */
	send_command (fd, "route");
	gtk_option_menu_set_history (GTK_OPTION_MENU (default_route_menu), atoi (get_answer (fd)));

	/* 
	 * get last incoming call from telmond on (hopefully) fixed port
   * 5001 
	 */
	if ((answer = get_lastcall(TELMOND_PORT))) {
		strcpy (answers[0], strtok (answer, " ")); /* date */
		strcpy (answers[1], strtok (NULL, " ")); /* time */
		strcpy (answers[2], strtok (NULL, " ")); /* caller nr. */
		strcpy (answers[3], strtok (NULL, " ")); /* callee nr. */
		sprintf (answers[4], "%s -> %s (%s %s)", answers[2], answers[3], answers[0], answers[1]);
		gtk_label_set_text (GTK_LABEL(lastcall_label), answers[4]);
	} else {
		gtk_label_set_text (GTK_LABEL(lastcall_label), _("none"));
	} 

	/* updating clist-window */
	if (!is_online && !was_online && !init_show_status)
		return;

	if (init_show_status)
		init_show_status = FALSE;

	/* freeze clists */
	gtk_clist_freeze (GTK_CLIST (clist));

	/* update status display for DSL (if DSL)*/
	if (pppoe) {
		send_command (fd, "status pppoe");
		if (strcmp (get_answer (fd), "Online") == 0) {
			/* we are online */			
			send_command (fd, "phone pppoe");
			gtk_clist_set_pixtext (GTK_CLIST (clist), j, 0, get_answer(fd), 5, online_icon, online_icon_mask);
			
		} else  {
			/* we are offline - don't show state information */
			gtk_clist_set_pixmap (GTK_CLIST (clist), j, 0, offline_icon, offline_icon_mask);
		}

		send_command (fd, "inout pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 1, get_answer (fd));
		
		send_command (fd, "ip pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 2, get_answer (fd));
		
		send_command (fd, "rate pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 3, get_bytes (atoi (strtok (get_answer (fd), " ")), buf));
		gtk_clist_set_text (GTK_CLIST (clist), j, 4, get_bytes (atoi (strtok (NULL, " ")), buf));

		send_command (fd, "online-time pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 5, get_answer (fd));
		
		send_command (fd, "time pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 6, get_answer (fd));
		
		send_command (fd, "chargetime pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 7, get_answer (fd));
		
		send_command (fd, "charge pppoe");
		gtk_clist_set_text (GTK_CLIST (clist), j, 8, get_answer (fd));

		j++; /* remember that we have a dsl-circuit */
	}

	/* update status display for ISDN channels */
	for(i=1; i<=channels; i++) {

		/* set online-status icons and connection name */
		sprintf (buf, "status %i", i);
		send_command (fd, buf);
		if (strcmp (get_answer (fd), "Online") == 0) { /* we are online on this channel */
			/* get connection name */
			sprintf (buf, "phone %i", i);
			send_command (fd, buf);
			gtk_clist_set_pixtext (GTK_CLIST (clist), (j+i-1), 0, get_answer (fd), 5,online_icon, online_icon_mask);
		} else { 
			gtk_clist_set_pixmap (GTK_CLIST (clist), (j+i-1), 0, offline_icon, offline_icon_mask);
		}

		sprintf (buf, "inout %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 1, get_answer (fd));

		sprintf (buf, "ip %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 2, get_answer (fd));

		sprintf (buf, "rate %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 3, get_bytes (atoi (strtok (get_answer (fd), " ")), buf));
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 4, get_bytes (atoi (strtok (NULL, " ")), buf));

		sprintf (buf, "online-time %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 5, get_answer (fd));

		sprintf (buf, "time %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 6, get_answer (fd));

		sprintf (buf, "chargetime %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 7, get_answer (fd));

		sprintf (buf, "charge %i", i);
		send_command (fd, buf);
		gtk_clist_set_text (GTK_CLIST (clist), (j+i-1), 8, get_answer (fd));

	}

	/* release clists */
	gtk_clist_thaw (GTK_CLIST (clist));

} /* status_display_update (gpointer window) */


/*----------------------------------------------------------------------------
 * called if connection was lost
 *----------------------------------------------------------------------------
 */
void fli4l_reconnect (int signo) {

	if (reconnect_dialog == NULL) {
		gtk_timeout_remove (timeout_tag);
		
		g_print ("Connection to %s lost.\n", host_name);
/* 		service_disconnect (fd); */

		reconnect_dialog = create_reconnect_dialog ();
		gtk_object_set_data (GTK_OBJECT (reconnect_dialog), "gtk_imonc", gtk_imonc);
		gtk_widget_show (reconnect_dialog);
	}
}

/*----------------------------------------------------------------------------
 * some helper functions
 *----------------------------------------------------------------------------
 */

/* converts int representing bytes into a nice formated string */
gchar *
get_bytes (gint bytes, gchar *buf)
{
	gfloat flobytes;

	if (bytes >= (1024*1024)) {
		flobytes = (bytes / 1024.0) / 1024.0 ;
		sprintf (buf, "%.1fM", flobytes);
		return buf;
	}

	if (bytes >= 1024) {
		flobytes = bytes / 1024.0;
		sprintf (buf, "%.1fK", flobytes);
		return buf;
	} 

	sprintf (buf, "%d", bytes);
	return buf;

} /* get_bytes (bytes, *buf) */

/* converts int representing seconds into a string in the format HH:MM:SS */
gchar *
get_timestr (gint time, gchar *buf)
{
	gint hour, min, sec;

	hour = time / 3600;
	min = (time - hour * 3600) / 60;
	sec = time - hour * 3600 - min * 60;

	if (hour < 10)
		sprintf (buf, "0%d", hour);
	else 
		sprintf (buf, "%d", hour);

	if (min < 10)
		sprintf (buf, "%s:0%d", buf, min);
	else 
		sprintf (buf, "%s:%2d", buf, min);

	if (sec < 10)
		sprintf (buf, "%s:0%d", buf, sec);
	else 
		sprintf (buf, "%s:%2d", buf, sec);

	return buf;
} /* get_timestr (time, *buf) */

/* converts int representing seconds into a string in the format "days HH:MM:SS" */
gchar *
get_datestr (gint time, gchar *buf)
{
	gint days, hour, min, sec;

	days = time / (3600 * 24);
	hour = (time - days * 3600 * 24) / 3600;
	min = (time - days * 3600 * 24 - hour * 3600) / 60;
	sec = time - days *3600 * 24 - hour * 3600 - min * 60;

	if (days > 0)
		sprintf (buf , "%dd %d", days, hour);
	else 
		sprintf (buf, "%d", hour);

	if (min < 10)
		sprintf (buf, "%s:0%d", buf, min);
	else 
		sprintf (buf, "%s:%2d", buf, min);

	if (sec < 10)
		sprintf (buf, "%s:0%d", buf, sec);
	else 
		sprintf (buf, "%s:%2d", buf, sec);

	return buf;
} /* get_datestr (time, *buf) */


/*----------------------------------------------------------------------------
 * dumps clist as csv to file
 *----------------------------------------------------------------------------
 */
void
csv_dump_clist (GtkCList *clist, FILE *fd)
{
	gint i, j;
	
	for (i=0; i<clist->columns; i++)
		{
			if (i>0)
				fprintf (fd, ",");
			fprintf (fd, "\"%s\"", gtk_clist_get_column_title (clist, i));
		}

	fprintf (fd, "\n");

	for (i=0; i<clist->rows; i++)
		{
			for (j=0; j<clist->columns; j++)
				{
					gchar *cellstr;
					if (j>0)
						fprintf (fd, ",");
					gtk_clist_get_text (clist, i, j, &cellstr);
					fprintf (fd,"\"%s\"", cellstr);
				}
			fprintf (fd, "\n");
		}
}
