#!/bin/sh
#------------------------------------------------------------------------------
# /etc/portfw.sh - ip port forwarding
#------------------------------------------------------------------------------
# ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION
#                  NEW ARGUMENTS, SEE BELOW !!!
# ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION ATTENTION
#------------------------------------------------------------------------------
# Called by /etc/ppp/ip-up everytime a new connection will be established
#
# Either ipportfw or ipautofw can be used
#------------------------------------------------------------------------------
# Usage of portfw (NEW):
#
# add entry:
#   ipmasqadm portfw -a -P PROTO -L LADDR LPORT -R HIDDENHOST RPORT [-p PREF]
# delete entry:
#   ipmasqadm portfw -d -P PROTO -L LADDR LPORT [-R HIDDENHOST RPORT]
# clear table:
#   ipmasqadm portfw -f
# list table:
#   ipmasqadm portfw -l
# no names:
#   ipmasqadm portfw <args> -n
#
#   PROTO       protocol, can be "tcp" or "udp"
#   LADDR       local interface receiving packets to be forwarded
#   LPORT       port being redirected
#   HIDDENHOST  ip address of your lan computer using the application
#   RPORT       the port being redirected to
#   PREF        preference level (load balancing, default=10)
#------------------------------------------------------------------------------
# Usage of autofw (NEW):
#
# add entry:
#   ipmasqadm autofw -A -r PROTO LOW HIGH [-h HIDDENHOST] [-c ctltype ctlport]
# clear table (flush):
#   ipmasqadm autofw -F
#
#   PROTO       protocol, can be "tcp" or "udp"
#   LOW         lowest port of range
#   HIGH        highest port of range
#   HIDDENHOST  ip address of your lan computer using the application
#------------------------------------------------------------------------------
# Here you will find hints to get your application run:
#
#                      http://www.tsmservices.com/masq/
#------------------------------------------------------------------------------
# Creation:     08.09.2000  fm
# Last Update:  07.09.2001  tg
#------------------------------------------------------------------------------
# You can access the IP of the ppp device with $ip
#------------------------------------------------------------------------------
# Define your PCs to forward ports to:
#------------------------------------------------------------------------------
hiddenhost1=192.168.6.2                         # ip of PC in LAN: change here!
hiddenhost2=192.168.6.3                         # ip of PC in LAN: change here!
hiddenhost3=192.168.6.4                         # ip of PC in LAN: change here!
hiddenhost4=192.168.6.5                         # ip of PC in LAN: change here!

#------------------------------------------------------------------------------
# example for ftp (internal ftp server, access from outside):
#
# Don't forget to correct the firewall ports in base.txt, see FW_N_PORTS
# and FW_PORT_x
#
# and to enable the ftp-Masquerading module in config.txt:
#   MASQ_MODULE_1='ftp'                         # masquerading module: ftp
#------------------------------------------------------------------------------
#/usr/sbin/ipmasqadm autofw -A -v -r tcp 21 21 -h $hiddenhost1

#------------------------------------------------------------------------------
# example for Battlecom (using ipautofw):
#------------------------------------------------------------------------------
#/usr/sbin/ipmasqadm autofw -A -v -r udp  2300  2400 -h $hiddenhost1
#/usr/sbin/ipmasqadm autofw -A -v -r tcp  2300  2400 -h $hiddenhost1
#/usr/sbin/ipmasqadm autofw -A -v -r tcp 47624 47624 -h $hiddenhost1
#/usr/sbin/ipmasqadm autofw -A -v -r udp 47624 47624 -h $hiddenhost1
#/usr/sbin/ipmasqadm autofw -A -v -r udp 28800 28900 -h $hiddenhost1

#------------------------------------------------------------------------------
# example for Napster:
# forward napster connections to 4 different intern PCs:
#
# PLEASE READ
#    http://www.linuxdoc.org/HOWTO/mini/IPMasquerading+Napster-4.html
#------------------------------------------------------------------------------
#nap1=6702                                           # napster port of PC 1
#nap2=6703                                           # napster port of PC 2
#nap3=6704                                           # napster port of PC 3
#nap4=6705                                           # napster port of PC 4
#/usr/sbin/ipmasqadm portfw -a -P tcp -L $ip $nap1 -R $hiddenhost1 $nap1
#/usr/sbin/ipmasqadm portfw -a -P tcp -L $ip $nap2 -R $hiddenhost2 $nap2
#/usr/sbin/ipmasqadm portfw -a -P tcp -L $ip $nap3 -R $hiddenhost3 $nap3
#/usr/sbin/ipmasqadm portfw -a -P tcp -L $ip $nap4 -R $hiddenhost4 $nap4

#------------------------------------------------------------------------------
# END
#------------------------------------------------------------------------------
