/*----------------------------------------------------------------------------
 *  mini-login.c   - mini version of login
 *
 *  Copyright (c) 2000-2001 Frank Meyer <frank@fli4l.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Creation:	    06.07.2000  fm
 *  Last Update:    09.09.2001  fm
 *----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <pwd.h>
#include <shadow.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <termio.h>
#include <fcntl.h>

static struct termio    oldmode;
static struct termio    newmode;

int
main (void)
{
    char    buf[16];
    char    salt[3];
    char *  pass;
    char *  p;
    struct  spwd * sp;
    int	    login_correct = 0;
    FILE *  fp;
    FILE *  fpv;

    putchar ('\n');

    fp = fopen ("/etc/issue", "r");

    if (fp)
    {
	int ch;

	while ((ch = getc (fp)) != EOF)
	{
	    if (ch == '%')
	    {
		ch = getc (fp);

		switch (ch)
		{
		    case 'v':
			fpv = fopen ("/etc/version", "r");

			if (fpv)
			{
			    while ((ch = getc (fpv)) != EOF &&
				   ch != '\r' && ch != '\n')
			    {
				putchar (ch);
			    }
			    fclose (fpv);
			}
			else
			{
			    fputs ("<unknown version>", stdout);
			}
			break;

		    default:
			putchar ('%');
			break;
		}
	    }

	    if (ch != '\r')
	    {
		putchar (ch);
	    }
	}

	fclose (fp);

	if (ch != '\n')		/* Windows editors don't add nl at eof :-(  */
	{
	    putchar ('\n');
	}

	putchar ('\n');
    }

    sp = getspnam ("root");

    if (! sp)
    {
	fputs ("cannot get passwd entry for root\n", stderr);
	exit (1);
    }

    pass = sp->sp_pwdp;

    if (pass)
    {
	int	len = strlen (pass) + 1;
	char *	p   = malloc (len);
	strcpy (p, pass);
	pass = p;
    }

    endspent ();

    if (pass && *pass)
    {
	(void) ioctl (0, TCGETA, &oldmode);
	(void) ioctl (0, TCGETA, &newmode);

	newmode.c_iflag |= ICRNL;
	newmode.c_iflag &= ~INLCR;

	newmode.c_oflag |= ONLCR;
	newmode.c_oflag &= ~ONLRET;

	newmode.c_lflag &= ~ICANON;
	newmode.c_lflag &= ~ISIG;
	newmode.c_lflag &= ~ECHO;

	(void) ioctl (0, TCSETAW, &newmode);

	fputs ("Password: ", stdout);

	p = fgets (buf, 15, stdin);
	putchar ('\n');

	if (p)
	{
	    p = strchr (buf, '\r');

	    if (p)
	    {
		*p = '\0';
	    }

	    p = strchr (buf, '\n');

	    if (p)
	    {
		*p = '\0';
	    }

	    salt[2] = '\0';
	    strncpy (salt, pass, 2);

	    if (! strcmp (pass, crypt (buf, salt)))
	    {
		login_correct = 1;
	    }
	}

	(void) ioctl (0, TCSETAW, &oldmode);
	fflush (stdout);
    }
    else
    {
	login_correct = 1;
    }

    if (login_correct)
    {
	char *	argv[2];

	argv[0] = "-sh";
	argv[1] = (char *) NULL;

	execvp ("/bin/sh", argv);
	fputs ("exec error: cannot execute /bin/sh", stderr);
	sleep (2);
	exit (1);
    }

    if (*buf)
    {
	fputs ("login incorrect\n", stderr);
	sleep (2);
    }

    return (1);
} /* main (argc, argv) */
