/*----------------------------------------------------------------------------
 *  mkfli4l.c   - create opt file list for fli4l
 *
 *  Compilation by Makefile (called by ../mkfloppy.sh)
 *
 *  Copyright (c) 2000-2001 Frank Meyer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Creation:       03.05.2000  fm
 *  Last Update:    18.09.2001  fm
 *----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>

#ifdef WINDOWS
#include <direct.h>
#else
#include <dirent.h>
#endif

#include <string.h>
#include "var.h"
#include "check.h"

#ifndef TRUE
# define TRUE               1
# define FALSE              0
#endif

#define OK		    0
#define ERR		    (-1)

#define BUF_SIZE	    1024

#define MAX_ETH_DRV	    8			    /* max. 8 eth drivers   */
#define MAX_SCSI_DRV	    8			    /* max. 8 scsi drivers  */

#ifdef WINDOWS
#  define strcasecmp stricmp
#endif


int		eth_drv_found[MAX_ETH_DRV];
int		scsi_drv_found[MAX_SCSI_DRV];
int		n_eth_drv;		/* number of ethernet drivers       */
int		n_scsi_drv;		/* number of scsi drivers	    */
int		n_masq_modules;		/* number of masquerading modules   */
int		isdn_type;		/* 0-99 HiSax, 101-102 AVMB1        */

#define MAX_PACKAGES	64
char *		packages[MAX_PACKAGES];
int		n_packages = 0;

FILE *		logfp;

static void	make_zip_list (char *);
static void	add_to_zip_list (FILE *, char *);

void		fatal_exit (char *);

/*----------------------------------------------------------------------------
 *  main (int argc, char ** argv)
 *----------------------------------------------------------------------------
 */
int
main (int argc, char ** argv)
{
    DIR *	    dirp;
    struct dirent * dp;
    char	    config_file[BUF_SIZE];
    char	    check_file[BUF_SIZE];
    char	    package[BUF_SIZE];
    char *	    config_dir;
    char *	    check_dir;
    char *	    opt_file;
    char *	    rc_file;
    char *	    zip_list_file;
    char *	    logfile;
    char *	    p;
    int		    len;

    config_dir	    = "config";
    check_dir	    = "check";
    opt_file        = "opt/opt.txt";
    rc_file         = "opt/etc/rc.cfg";
    zip_list_file   = "opt/opt.tmp";
    logfile	    = "img/mkfli4l.log";

    logfp = fopen (logfile, "w");

    if (! logfp)
    {
	fatal_exit (logfile);
    }

    if (argc > 2)
    {
	fprintf (stderr, "usage: %s [config-directroy]\n", argv[0]);
	fprintf (logfp, "usage: %s [config-directroy]\n", argv[0]);
	fatal_exit ((char *) NULL);
    }

    if (argc == 2)
    {
	config_dir = argv[1];
    }

    sprintf (config_file, "%s/base.txt", config_dir);	/* begin with base  */

    if (access (config_file, 0) != 0)
    {
	fatal_exit (config_file);
    }

    if (read_config_file (config_file, "base") != OK)
    {
	fatal_exit ((char *) NULL);
    }

    packages[n_packages++] = strsave ("base");

    sprintf (check_file, "%s/%s.txt", check_dir, "base");

    if (read_check_file (check_file) != OK)
    {
	fatal_exit ((char *) NULL);
    }

    sprintf (config_file, "%s/dsl.txt", config_dir);	/* cont with dsl    */

    if (access (config_file, 0) == 0)			/* it's optional    */
    {
	if (read_config_file (config_file, "dsl") != OK)
	{
	    fatal_exit ((char *) NULL);
	}
	packages[n_packages++] = strsave ("dsl");

	sprintf (check_file, "%s/%s.txt", check_dir, "dsl");

	if (read_check_file (check_file) != OK)
	{
	    fatal_exit ((char *) NULL);
	}
    }

    sprintf (config_file, "%s/isdn.txt", config_dir);	/* cont with isdn   */

    if (access (config_file, 0) == 0)			/* it's optional    */
    {
	if (read_config_file (config_file, "isdn") != OK)
	{
	    fatal_exit ((char *) NULL);
	}
	packages[n_packages++] = strsave ("isdn");

	sprintf (check_file, "%s/%s.txt", check_dir, "isdn");

	if (read_check_file (check_file) != OK)
	{
	    fatal_exit ((char *) NULL);
	}
    }

    dirp = opendir (config_dir);

    if (! dirp)
    {
	fatal_exit (config_dir);
    }

    while ((dp = readdir (dirp)) != (struct dirent *) NULL)
    {
	len = strlen (dp->d_name);

	if (len > 4 && ! strcasecmp (dp->d_name + len - 4, ".txt") &&
	    strcasecmp (dp->d_name, "base.txt") &&
	    strcasecmp (dp->d_name, "dsl.txt") &&
	    strcasecmp (dp->d_name, "isdn.txt"))
	{
	    if (! strcasecmp (dp->d_name, "config.txt"))
	    {
		fprintf (stderr,
			 "Error: %s/%s is an old style config file. READ DOCUMENTATION!\n",
			 config_dir, dp->d_name);
		fprintf (logfp, 
			 "Error: %s/%s is an old style config file. READ DOCUMENTATION!\n",
			 config_dir, dp->d_name);
		fatal_exit ((char *) NULL);
	    }

	    sprintf (config_file, "%s/%s", config_dir, dp->d_name);
	    strcpy (package, dp->d_name);
	    *(package + len - 4) = '\0';

	    if (read_config_file (config_file, package) != OK)
	    {
		fatal_exit ((char *) NULL);
	    }
	    packages[n_packages++] = strsave (package);

	    sprintf (check_file, "%s/%s.txt", check_dir, package);

	    if (read_check_file (check_file) != OK)
	    {
		fatal_exit ((char *) NULL);
	    }
	}
    }

    (void) closedir (dirp);

    if (check_all_variables () != OK)
    {
	fatal_exit ((char *) NULL);
    }

    p = get_variable ("ETH_DRV_N");

    if (! p)
    {
	fprintf (stderr, "%s: Variable ETH_DRV_N not set\n", config_file);
	fprintf (logfp, "%s: Variable ETH_DRV_N not set\n", config_file);
	fatal_exit ((char *) NULL);
    }

    n_eth_drv = atoi (p);

    if (n_eth_drv <= 0)
    {
	fprintf (stderr, "%s: Variable ETH_DRV_N must be positive\n",
		 config_file);
	fprintf (logfp, "%s: Variable ETH_DRV_N must be positive\n",
		 config_file);
	fatal_exit ((char *) NULL);
    }

    p = get_variable ("OPT_SCSI");

    if (p && ! strcmp (p, "yes"))
    {
	p = get_variable ("SCSI_DRV_N");

	if (! p)
	{
	    fprintf (stderr, "%s: Variable SCSI_DRV_N not set\n", config_file);
	    fprintf (logfp, "%s: Variable SCSI_DRV_N not set\n", config_file);
	    fatal_exit ((char *) NULL);
	}

	n_scsi_drv = atoi (p);

	if (n_scsi_drv < 0)
	{
	    fprintf (stderr, "%s: Variable SCSI_DRV_N must be positive or 0\n",
		     config_file);
	    fprintf (logfp, "%s: Variable SCSI_DRV_N must be positive or 0\n",
		     config_file);
	    fatal_exit ((char *) NULL);
	}
    }
    else
    {
	n_scsi_drv = 0;
    }

    p = get_variable ("MASQ_MODULE_N");

    if (! p)
    {
	fprintf (stderr, "%s: Variable MASQ_MODULE_N not set\n",
		 config_file);
	fprintf (logfp, "%s: Variable MASQ_MODULE_N not set\n",
		 config_file);
	fatal_exit ((char *) NULL);
    }

    n_masq_modules = atoi (p);

    if (n_masq_modules < 0)
    {
	fprintf (stderr,
		 "%s: Variable MASQ_MODULE_N must not be negative\n",
		 config_file);
	fprintf (logfp,
		 "%s: Variable MASQ_MODULE_N must not be negative\n",
		 config_file);
	fatal_exit ((char *) NULL);
    }

    make_zip_list (zip_list_file);

    if (dump_variables (rc_file) != OK)
    {
	fatal_exit ((char *) NULL);
    }

    fclose (logfp);
    (void) remove (logfile);

    return (0);
} /* main (argc, argv) */

/*----------------------------------------------------------------------------
 *  convert_to_upper (char * str)
 *----------------------------------------------------------------------------
 */
static char *
convert_to_upper (char * str)
{
    static char *   rtc;
    unsigned char * s;
    unsigned char * r;

    if (! str)
    {
	return ((char *) NULL);
    }

    if (rtc)
    {
	free (rtc);
    }

    rtc = (char *) malloc (strlen (str) + 1);

    if (! rtc)
    {
	return ((char *) NULL);
    }

    for (s = (unsigned char *) str, r = (unsigned char *) rtc; *s; s++, r++)
    {
	if (isascii (*s) && islower (*s))
	{
	    *r = toupper (*s);
	}
	else
	{
	    *r = *s;
	}
    }

    *r = '\0';

    return (rtc);
} /* convert_to_upper (str) */


/*----------------------------------------------------------------------------
 *  make_zip_list (char * zip_list_file)
 *----------------------------------------------------------------------------
 */
static void
make_zip_list (char * zip_list_file)
{
    char        buf[BUF_SIZE];
    char        tmp_buf[BUF_SIZE];
    char        opt_file[BUF_SIZE];
    char        context[BUF_SIZE];
    char        name[BUF_SIZE];
    char        filename[BUF_SIZE];
    FILE *      opt_fp;
    FILE *      zip_list_fp;
    char *      drv_name;
    char *      p;
    char *      tmp_p;
    int         pkg_idx;
    int         idx = 0;

    zip_list_fp = fopen (zip_list_file, "wb");

    if (! zip_list_fp)
    {
	fatal_exit (zip_list_file);
    }

    for (pkg_idx = 0; pkg_idx < n_packages; pkg_idx++)
    {
	sprintf (opt_file, "opt/%s.txt", packages[pkg_idx]);

	opt_fp = fopen (opt_file, "r");

	if (! opt_fp)
	{
	    fatal_exit (opt_file);
	}

	while (fgets (buf, BUF_SIZE, opt_fp))
	{
	    p = strchr (buf, '#');			    /* strip comments   */

	    if (p)
	    {
		*p = '\0';
	    }

	    for (p = buf; *p; p++)
	    {
		if (*p != ' ' && *p != '\t')
		{
		    break;
		}
	    }

	    if (*p == '\0' || *p == '\r' || *p == '\n')
	    {
		continue;
	    }

	    if (sscanf (p, "%s %s %s", context, name, filename) != 3)
	    {
		fprintf (stderr, "%s: syntax error: %d '%s'\n",
			 opt_file, *p, p);
		fprintf (logfp, "%s: syntax error: %d '%s'\n",
			 opt_file, *p, p);
		fatal_exit ((char *) NULL);
	    }

	    if (! strcmp (context, "gen"))
	    {
		add_to_zip_list (zip_list_fp, filename);
	    }
	    else if (! strcmp (context, "dns"))
	    {
		char *  start_dns = get_variable ("START_DNS");

		if (start_dns)
		{
		    if (! strcmp (start_dns, "yes") && ! strcmp (name, "ens"))
		    {
			add_to_zip_list (zip_list_fp, filename);
		    }
		}
		else
		{
		    fprintf (stderr, "Variable START_DNS not set\n");
		    fprintf (logfp, "Variable START_DNS not set\n");
		    fatal_exit ((char *) NULL);
		}
	    }
	    else if (! strcmp (context, "masq"))
	    {
		for (idx = 1; idx <= n_masq_modules; idx++)
		{
		    sprintf (tmp_buf, "MASQ_MODULE_%d", idx);

		    drv_name = get_variable (tmp_buf);

		    if (! drv_name)
		    {
			fprintf (stderr, "Variable %s not set\n", tmp_buf);
			fprintf (logfp, "Variable %s not set\n", tmp_buf);
			fatal_exit ((char *) NULL);
		    }

		    tmp_p = drv_name; 

		    while ((tmp_p = strchr (tmp_p, '\t')) != (char *) NULL)
		    {
			*tmp_p = ' ';
		    }

		    if (! strcmp (drv_name, name))
		    {
			add_to_zip_list (zip_list_fp, filename);
			break;
		    }
		}
	    }
	    else if (! strcmp (context, "eth"))
	    {
		for (idx = 1; idx <= n_eth_drv; idx++)
		{
		    sprintf (tmp_buf, "ETH_DRV_%d", idx);

		    drv_name = get_variable (tmp_buf);

		    if (! drv_name)
		    {
			fprintf (stderr, "Variable %s not set\n", tmp_buf);
			fprintf (logfp, "Variable %s not set\n", tmp_buf);
			fatal_exit ((char *) NULL);
		    }

		    tmp_p = drv_name; 

		    while ((tmp_p = strchr (tmp_p, '\t')) != (char *) NULL)
		    {
			*tmp_p = ' ';
		    }

		    tmp_p = strchr (drv_name, ' ');

		    if (tmp_p)
		    {
			drv_name = tmp_p + 1;

			while (*drv_name == ' ')
			{
			    drv_name++;
			}
		    }

		    if (! strcmp (drv_name, name))
		    {
			eth_drv_found[idx - 1] = TRUE;
			add_to_zip_list (zip_list_fp, filename);
			/* no break, allow drivers to be loaded twice! */
		    }
		}
	    }
	    else if (! strcmp (context, "scsi"))
	    {
		for (idx = 1; idx <= n_scsi_drv; idx++)
		{
		    sprintf (tmp_buf, "SCSI_DRV_%d", idx);

		    drv_name = get_variable (tmp_buf);

		    if (! drv_name)
		    {
			fprintf (stderr, "Variable %s not set\n", tmp_buf);
			fprintf (logfp, "Variable %s not set\n", tmp_buf);
			fatal_exit ((char *) NULL);
		    }

		    tmp_p = drv_name; 

		    while ((tmp_p = strchr (tmp_p, '\t')) != (char *) NULL)
		    {
			*tmp_p = ' ';
		    }

		    tmp_p = strchr (drv_name, ' ');

		    if (tmp_p)
		    {
			drv_name = tmp_p + 1;

			while (*drv_name == ' ')
			{
			    drv_name++;
			}
		    }

		    if (! strcmp (drv_name, name))
		    {
			scsi_drv_found[idx - 1] = TRUE;
			add_to_zip_list (zip_list_fp, filename);
			break;
		    }
		}
	    }
	    else if (! strcmp (context, "isdn"))
	    {
		char *  opt_isdn;
		int	type;

		opt_isdn = get_variable ("OPT_ISDN");

		if (opt_isdn && ! strcmp (opt_isdn, "yes"))
		{
		    drv_name = get_variable ("ISDN_TYPE");

		    if (! drv_name)
		    {
			fprintf (stderr, "Variable ISDN_TYPE not set\n");
			fprintf (logfp, "Variable ISDN_TYPE not set\n");
			fatal_exit ((char *) NULL);
		    }

		    if (*drv_name)
		    {
			type = atoi (drv_name);

			if (! strcmp (name, "gen"))
			{
			    add_to_zip_list (zip_list_fp, filename);
			}
			else if (type > 0 && type < 100)
			{
			    if (! strcmp (name, "hisax"))
			    {
				add_to_zip_list (zip_list_fp, filename);
			    }
			    else if (! strncmp (name, "hisax-", 6) &&
				     ! strcmp (name + 6, drv_name))
			    {
				add_to_zip_list (zip_list_fp, filename);
			    }
			}
			else if ((type == 101 || type == 102) && ! strcmp (name, "avmb1"))
			{
			    add_to_zip_list (zip_list_fp, filename);
			}
			else if ((type == 201) && ! strcmp (name, "icn"))
			{
			    add_to_zip_list (zip_list_fp, filename);
			}
		    }
		}
	    }
	    else if (! strcmp (context, "opt"))
	    {
		char *  option;

		sprintf (buf, "OPT_%s", convert_to_upper (name));

		option = get_variable (buf);		    /* e.g. OPT_TOOLS   */

		if (option)
		{
		    if (! strcmp (option, "yes"))
		    {
			add_to_zip_list (zip_list_fp, filename);
		    }
		}
		else
		{
		    fprintf (stderr, "warning: variable %s not set\n", buf);
		    fprintf (logfp, "warning: variable %s not set\n", buf);
		}
	    }
	    else
	    {
		fprintf (stderr, "%s: unknown context: %s\n", opt_file, context);
		fprintf (logfp, "%s: unknown context: %s\n", opt_file, context);
		fatal_exit ((char *) NULL);
	    }
	}

	fclose (opt_fp);
    }

    fclose (zip_list_fp);

    for (idx = 1; idx <= n_eth_drv; idx++)
    {
	if (! eth_drv_found[idx - 1])
	{
	    sprintf (tmp_buf, "ETH_DRV_%d", idx);
	    drv_name = get_variable (tmp_buf);
	    fprintf (stderr, "%s: cannot find kernel module for: %s='%s'\n",
		     opt_file, tmp_buf, drv_name);
	    fprintf (logfp, "%s: cannot find kernel module for: %s='%s'\n",
		     opt_file, tmp_buf, drv_name);
	    fatal_exit ((char *) NULL);
	}
    }

    for (idx = 1; idx <= n_scsi_drv; idx++)
    {
	if (! scsi_drv_found[idx - 1])
	{
	    sprintf (tmp_buf, "SCSI_DRV_%d", idx);
	    drv_name = get_variable (tmp_buf);
	    fprintf (stderr, "%s: cannot find kernel module for: %s='%s'\n",
		     opt_file, tmp_buf, drv_name);
	    fprintf (logfp, "%s: cannot find kernel module for: %s='%s'\n",
		     opt_file, tmp_buf, drv_name);
	    fatal_exit ((char *) NULL);
	}
    }

    return;
} /* make_zip_list (zip_list_file) */


/*----------------------------------------------------------------------------
 *  add_to_zip_list (FILE * fp, char * filename)
 *----------------------------------------------------------------------------
 */
static void
add_to_zip_list (FILE * fp, char * filename)
{
    static char *   filenames[MAX_FILENAMES];
    static int	    n_filenames;
    char	    buf[BUF_SIZE];
    int		    i;

    for (i = 0; i < n_filenames; i++)		/* check, if file already   */
    {						/* added ...		    */
	if (! strcmp (filenames[i], filename))	/* added already before?    */
	{					/* yes, ignore it and	    */
	    return;				/* return		    */
	}
    }

    sprintf (buf, "opt/%s", filename);

    if (access (buf, 0) != 0)
    {
	fatal_exit (filename);
    }

    if (n_filenames >= MAX_FILENAMES)
    {
	fprintf (stderr, "sorry, too many files, max is %d\n", MAX_FILENAMES);
	fprintf (logfp, "sorry, too many files, max is %d\n", MAX_FILENAMES);
	fatal_exit ((char *) NULL);
    }

    filenames[n_filenames] = strsave (filename);
    n_filenames++;

    fprintf (fp, "%s\n", filename);

    return;
} /* add_to_zip_list (fp, filename) */

void
log_perror (char * s)
{
    perror (s);
    fprintf (logfp, "%s: cannot open\n", s);
    return;
}

void
fatal_exit (char * s)
{
    if (s)
    {
	log_perror (s);
    }

    if (logfp)
    {
	fclose (logfp);
    }

    exit (1);
}
